.\"
.\"  $Id: Collection.n,v 1.4 1994/03/01 23:01:21 dta Exp $
.\"  $Source: /cvs/lib/odmg/Collection.n,v $
.\"
.TH Collection Odmg3 "02/06/94" "ODMG-93" "Class Manual"
.SH NAME
Collection \- Abstract base type for all collection classes.
.SH SYNOPSIS
#include <Odmg/Collection.h>
.SH DESCRIPTION
The Collection class is a abstract base type for all of the collection classes
in the ODMG object model.  It contains a standard interface to a collection
that must be supported by all derived classes.
.P
A error message will be printed if a method in a derived class is
called that should be treated as pure virtual and does not have a
implmentation in the derived class.
.SH METHODS
.IP "Position cardinality (void) const;"
Returns the number of valid elements in the collection.
This number is maintained in the protected m_count field in this class.
.IP
\fICreated a type called Position to be used when indexing elements.\fR
\fIMade the method const.\fR
.IP "boolean empty (void) const;"
Returns TRUE if the collection is empty.
.IP
\fBName should be changed to is_empty.\fR
.IP "virtual boolean ordered (void) const;"
Returns TRUE if the collection is ordered.
.IP
\fIThis returns the protected field m_is_ordererd.
.IP
\fIDefault value is FALSE.\fR
.IP
\fBName should be changed to is_ordered.\fR
.IP "virtual boolean allows_duplicates (void) const;"
Returns TRUE if the collection allows duplicate members.
.IP
\fIThis returns the protected field m_allows_duplicates.
.IP
\fIDefault value is TRUE.\fR
.IP "Collection (void);"
Constructor for a collection.
.IP
\fIThe constructor is protected because this class is a abstract base class.\fR
.IP "virtual ~Collection (void);"
Virtual destructor for collection classes.
It does not delete the elements.
.IP
\fIMade the destructor virtual\fR
.IP "boolean operator == (Ref <Collection <T> >) const;"
Performs a shallow comparison of all objects within the collection.
Collections are the same if they contain the same references
in the same order.
.IP "boolean operator != (Ref <Collection <T> >) const;"
Performs a shallow comparison of all objects within the collection.
Returns TRUE if the collections are not equal.
.IP "Collection <T> &operator = (Ref <Collection <T> >);"
Does a assignment from one collection to the other
after removing all existing references from the collection.
After the assignment the collections are equal.
.IP "virtual Ref <Collection <T> > copy (void) const;"
Creates a new collection and copies all of the elements into it.
After the copy the two collections should be equal.
.IP
\fBTreat like pure virtual.\fR
.IP "virtual Ref <Iterator <T> > create_iterator (boolean stable = FALSE) const;"
Creates a iterator object for this collection.
It is up to the user to delete the iterator.
.IP
\fIChanged the return type to a reference.\fR
.IP
\fBArgument stable is not currently used.\fR
.IP "virtual void insert_element (Ref <T>);"
Inserts a element into the collection.
.IP
\fBTreat like pure virtual.\fR
.IP "void operator += (Ref <T>);"
Calls insert_element.
.IP "virtual void remove_element (Ref <T>);"
Removes a element from the collection.
.IP
\fBTreat like pure virtual.\fR
.IP "void operator -= (Ref <T>);"
Calls remove_element.
.IP "virtual void remove_element_at (Iterator <T> &);"
Removes a element from the collection based on the current position of the iterator.
.IP
\fIRemoved const for iterator because the iterator is modified
to point to the previous element in the collection.\fR
.IP "virtual void replace_element_at (Ref <T>, const Iterator<T> &);"
Replaces the element based on the position of the iterator with the new element.
.IP "virtual Ref <T> retrieve_element_at (const Iterator<T> &);"
Retrieve element from the current position of the iterator.
.IP "virtual Ref <T> select_element (const char *predicate);"
Select a element based on the predicate passed in.
.IP
\fBNot implemented.\fR
.IP "virtual Iterator <T> select (Iterator <T>, const char *predicate);"
Retrieve a iterator based on the predicate passed in.
.IP
\fBAdded the iterator argument.
.IP
\fBNot implemented.\fR
.IP "virtual int query (Ref <Collection<T> >&, const char *predicate);"
Create a new collection of objects based on the predicate.  Returns
the number of elements in the new collection.
.IP
\fBNot implemented.\fR
.IP "virtual boolean exists_element (const char *predicate) const;"
Returns TRUE if any elements in the collection match the predicate.
.IP
\fBNot implemented.\fR
.IP "virtual boolean contains_element (Ref <T>) const;"
Returns TRUE if the element exists in the collection.
.SH ADDITIONS
.IP "Ref <Collection <T> > _union (const Ref <Collection <T> >) const;"
Creates a new collection that is the union of the two collections.
.IP "Ref <Collection <T> > intersection (const Ref <Collection <T> >) const;"
Creates a new collection that is the intersection of the two collections.
.IP "Ref <Collection <T> > difference (const Ref <Collection <T> >) const;"
Creates a new collection that is the difference of the two collections.
.IP "Ref <Collection <T> > operator + (const Ref <Collection <T> >) const;"
Calls _union.
.IP "Ref <Collection <T> > operator * (const Ref <Collection <T> >) const;"
Calls intersection.
.IP "Ref <Collection <T> > operator - (const Ref <Collection <T> >) const;"
Calls difference.
.IP "Ref <Collection <T> > operator += (const Ref <Collection <T> >);"
Makes the union of this collection and the one passed in.
.IP "Ref <Collection <T> > operator *= (const Ref <Collection <T> >);"
Makes the intersection of this collection and the one passed in.
.IP "Ref <Collection <T> > operator -= (const Ref <Collection <T> >);"
Makes the difference of this collection and the one passed in.
.IP "boolean is_subset_of (const Ref <Collection <T> >) const;"
Returns TRUE if the collection is a subset of the collection passed.
.IP "boolean is_proper_subset_of (const Ref <Collection <T> >) const;"
Returns TRUE if the collection is a proper subset of the collection passed.
A proper subset has the same number of elements as the set?
.IP "boolean is_superset_of (const Ref <Collection <T> >) const;"
Returns TRUE if the collection is a supoerset of the collection passed.
.IP "boolean is_proper_superset_of (const Ref <Collection <T> >) const;"
Returns TRUE if the collection is a proper superset of the collection passed.
A proper superset has the same number of elements as the set?
.IP "virtual Ref <T> retrieve_element_at (const Position);"
Returns the element at the specified position.
.IP "void remove_all (void);"
Removes all of the references from a collection.
.IP "void delete_all (void);"
Deletes all of the references in a collection then
removes them.
.IP "virtual Ref <Collection <T> > create (void) const;"
Creates a new collection of the same type.
.IP
\fBTreat like pure virtual.\fR
.SH "SEE ALSO"
Odmg (Odmg3)
