/*******************************<+>***************************
 **                             DTA
 *************************************************************
 **
 **  $Id: List.h,v 1.11 1994/03/01 23:01:23 dta Exp $
 **
 **  $Source: /cvs/lib/odmg/List.h,v $
 **
 **  What @(#):
 **
 **  Author: Dale T. Anderson
 **
 *******************************<+>***************************/
 
#ifndef List_h
#define List_h

#include <Odmg/Collection.h>
#include <Odmg/ListP.h>

template <class T> class List;

template <class T>
class List :
    public Collection <T>, 
    public ListNode <T>
{
	virtual Ref <Collection <T> > create (void) const;

	ListNode <T> &get_node_at (const Position position);

	List (const Ref <List <T> > list);

    public:

	List (void);
	virtual ~List (void);

	Ref <T> operator[] (const Position);
	virtual Ref <T> retrieve_element_at (const Position);

	void replace_element_at (Ref <T>, const Position);

	Ref <List <T> > concat (Ref <List <T> >);
	Ref <List <T> > operator + (Ref <List <T> >);

	void append (Ref <List <T> >);
	void operator += (Ref <List <T> >);

	void insert_element_after (Ref <T>, const Position);
	void insert_element_before (Ref <T>, const Position);
	void insert_element_first (Ref <T>);
	void insert_element_last (Ref <T>);

	void remove_element_at (const Position);
	void remove_first_element (void);
	void remove_last_element (void);

	Ref <T> retrieve_first_element (void);
	Ref <T> retrieve_last_element (void);

	Position find_element (Ref <T>) const;

	//
	// Collection methods
	//

	virtual void insert_element (Ref <T>);
	virtual void remove_element (Ref <T>);

	virtual void replace_element_at (Ref <T>, const Iterator <T> &);
};

//
// List methods
//

template <class T>
List <T>::List (void)
{
}

template <class T>
List <T>::~List (void)
{
    remove_all ();
}

template <class T>
Ref <T> List <T>::operator[] (const Position position)
{
    return retrieve_element_at (position);
}

template <class T>
Ref <T> List <T>::retrieve_element_at (const Position position)
{
    return get_node_at (position).get_ref ();
}

template <class T>
void List <T>::replace_element_at (Ref <T> ref, const Position position)
{
    ListNode <T> &node = get_node_at (position);
    node.set_ref (ref);
}

template <class T>
Ref <List <T> > List <T>::concat (Ref <List <T> > list)
{
    List <T> *list2 = (List <T> *) copy ();
    list2->append (list);
    return Ref <List <T> > (list2);
}

template <class T>
Ref <List <T> > List <T>::operator + (Ref <List <T> > list)
{
    return concat (list);
}

template <class T>
void List <T>::append (Ref <List <T> > list)
{
    Iterator <T> iter (this);
    Ref <T> ref;

    while (iter.next (ref))
	insert_element_last (ref);
}

template <class T>
void List <T>::operator += (Ref <List <T> > list)
{
    append (list);
}

template <class T>
void List <T>::insert_element_after (Ref <T> ref, const Position position)
{
    ListNode <T> &node = get_node_at (position);
    node.append_node (new ListNode <T> (ref));
    m_count++;
}

template <class T>
void List <T>::insert_element_before (Ref <T> ref, const Position position)
{
    ListNode <T> &node = get_node_at (position);
    node.insert_node (new ListNode <T> (ref));
    m_count++;
}

template <class T>
void List <T>::insert_element_first (Ref <T> ref)
{
    insert_element_before (ref, 0);
}

template <class T>
void List <T>::insert_element_last (Ref <T> ref)
{
    insert_element_after (ref, cardinality () - 1);
}

template <class T>
void List <T>::remove_element_at (const Position position)
{
    ListNode <T> *node = &get_node_at (position);
    delete node;
    m_count--;
}

template <class T>
void List <T>::remove_first_element (void)
{
    remove_element_at (0);
}

template <class T>
void List <T>::remove_last_element (void)
{
    remove_element_at (cardinality () - 1);
}

template <class T>
Ref <T> List <T>::retrieve_first_element (void)
{
    return retrieve_element_at (0);
}

template <class T>
Ref <T> List <T>::retrieve_last_element (void)
{
    return retrieve_element_at (cardinality () - 1);
}

template <class T>
Position List <T>::find_element (Ref <T> target) const
{
    Iterator<T> iter ((Collection <T> *) this);
    Ref <T> ref;

    for (Position i = 0; iter.next (ref); i++)
	if (ref == target)
	    return i;
    
    return INVALID_POSITION;
}

//
// Collection methods
//

template <class T>
void List <T>::insert_element (Ref <T> ref)
{
    insert_node (new ListNode <T> (ref));
    m_count++;
}

template <class T>
void List <T>::remove_element (Ref <T> ref)
{
    ListNode <T> *node = get_next_node ();

    while (node != this) {
	if (ref == node->get_ref ()) {
	    delete node;
	    m_count--;
	    break;
	}
	node = node->get_next_node ();
    }
}

template <class T>
void List <T>::replace_element_at (Ref <T> ref, const Iterator <T> & iter)
{
    ListNode <T> &node = get_node_at (iter.get_position ());
    node.set_ref (ref);
}

//
// Additions
//

template <class T>
ListNode <T> &List <T>::get_node_at (const Position position)
{
    if (m_count && position >= 0 && position < m_count) {
	if (position * 2 >= m_count) {
	    ListNode <T> *node = this->get_prev_node ();

	    for (Position i = 0; i < m_count - position - 1; i++)
		node = node->get_prev_node ();

	    return *node;
	} else {
	    ListNode <T> *node = this->get_next_node ();

	    for (Position i = 0; i < position; i++)
		node = node->get_next_node ();

	    return *node;
	}
    }

    return *this;
}

template <class T>
List <T>::List (const Ref <List <T> > list)
{
    m_is_ordered = list->ordered ();
    m_allows_duplicates = list->allows_duplicates ();
}

template <class T>
Ref <Collection <T> > List <T>::create (void) const
{
    return Ref <Collection <T> > (new List <T> (this));
}

#endif
