/*******************************<+>***************************
 **                             DTA
 *************************************************************
 **
 **  $Id: collection.C,v 1.6 1994/03/01 23:01:30 dta Exp $
 **
 **  $Source: /cvs/lib/odmg/test/collection.C,v $
 **
 **  What @(#):
 **
 **  Author: Dale T. Anderson
 **
 *******************************<+>***************************/
 
#include <stdarg.h>

#include "Test.h"

int print (const char *fmt, ...)
{
    va_list args;
    va_start (args, fmt);
    int status = 0;
    if (verbose) {
	status = vfprintf (stdout, fmt, args);
	fflush (stdout);
    }
    va_end (args);
    return status;
}

void PrintInfo (const char *str, Collection <CTest> &c)
{
    if (!verbose)
	return;

    print ("%s:", str);
    print ("\tcardinality %ld", c.cardinality ());
    print (", empty %d", c.empty ());
    print (", ordered %d", c.ordered ());
    print (", allows_duplicates %d\n", c.allows_duplicates ());
}

static void PrintContents (const char *str, Collection <CTest> &c)
{
    Iterator <CTest> *iter = c.create_iterator ();
    Ref <CTest> ref;

    print ("\t%-15s { ", str);

    while (iter->next (ref)) {
	if (verbose)
	    ref->Print ();

	if (c.retrieve_element_at (*iter) != ref)
	    print ("iter != ref!!!\n");

	print (" ");
    }

    print ("};\n");

    delete iter;
}

static void Stress (Collection <CTest> &c)
{
    print ("stress:\n");

    for (int i = 0; i < 100; i++) {
	switch (rand () & 0x1) {
	    case 0:
		if (!c.empty ()) {
		    Position n = rand () % c.cardinality ();
		    Ref <CTest> ref = c.retrieve_element_at (n);
		    if (verbose > 1)
			print ("remove %d at %d", ref->GetVal (), n);
		    CTest *test = ref;
		    c.remove_element (ref);
		    delete test;
		}
		break;
	    default:
		{
		Position n = rand () % 100;

		if (verbose > 1)
		    print ("insert %d at %d", n, c.cardinality ());

		Ref <CTest> ref (new CTest ((int) n));

		c.insert_element (ref);
		}
		break;
	}
	if (verbose > 1)
	    PrintContents ("", c);
    }
}

static void Compare (Ref <Collection <CTest> > &c1, Ref <Collection <CTest> > &c2)
{
    print ("    is equal             %d\n", c1 == c2);
    //print ("    not equal            %d\n", Ref <CTest> (c1) == Ref <CTest> (c2));
}

static void remove_element_at_test (Collection <CTest> &)
{
}

static void replace_element_at_test (Collection <CTest> &)
{
}

static void retrieve_element_at_test (Collection <CTest> &)
{
}

static void select_element_test (Collection <CTest> &, const char *)
{
}

static void select_test (Collection <CTest> &c, const char *predicate)
{
    print ("select test:\t");

    Iterator <CTest> iter1 (&c);
    Iterator <CTest> &iter2 = c.select (iter1, predicate);

    //Ref <CTest> ref (iter);
    //ref->Print ();
}

static void query_test (Collection <CTest> &c, const char *predicate)
{
    print ("query test:\t");

    Ref <Collection <CTest> > collection (NULL);

    if (c.query (collection, predicate) >= 0) {
	PrintContents ("collection", *collection);
    }
}

static void exists_element_test (Collection <CTest> &c, const char *predicate)
{
    print ("exists_element test:\t");
    print ("%s = %d\n", c.exists_element (predicate));
}

static void contains_element_test (Collection <CTest> &c1, Collection <CTest> &c2)
{
    Iterator <CTest> *iter = c1.create_iterator ();
    Ref <CTest> ref;

    print ("contains_element test:\n");

    PrintContents ("c1", c1);
    PrintContents ("c2", c2);

    print ("\tc1 contains c2");

    while (iter->next (ref))
	print (" %d", c2.contains_element (ref));

    print ("\n");

    delete iter;
}

//
// Additions
//

static void remove_all_test (Collection <CTest> &c)
{
    c.remove_all ();
}

static void delete_all_test (Collection <CTest> &c)
{
    c.delete_all ();
}

static void set2_test (Ref <Collection <CTest> > c1, Ref <Collection <CTest> > c2)
{
    PrintContents ("c1", *c1);
    PrintContents ("c2", *c2);
    print ("c1->is_subset_of (c2)           %d\n", c1->is_subset_of (c2));
    print ("c1->is_proper_subset_of (c2)    %d\n", c1->is_proper_subset_of (c2));
    print ("c1->is_superset_of (c2)         %d\n", c1->is_superset_of (c2));
    print ("c1->is_proper_superset_of (c2)  %d\n", c1->is_proper_superset_of (c2));
}

static void set_test (Collection <CTest> &c)
{
    Collection <CTest> *c1 = c.create ();
    Collection <CTest> *c2 = c.create ();

    Ref <CTest> r1 (new CTest (1));
    Ref <CTest> r2 (new CTest (2));
    Ref <CTest> r3 (new CTest (3));
    Ref <CTest> r4 (new CTest (4));
    Ref <CTest> r5 (new CTest (5));

    c1->insert_element (r1);
    c1->insert_element (r2);
    c1->insert_element (r3);

    c2->insert_element (r3);
    c2->insert_element (r4);
    c2->insert_element (r5);

    PrintInfo ("c1", *c1);
    PrintInfo ("c2", *c2);

    PrintContents ("c1", *c1);
    PrintContents ("c2", *c2);

    Collection <CTest> *c3 = *c1 + *c2;
    PrintContents ("c3 = c1 + c2", *c3);

    Collection <CTest> *c4 = *c1 * *c2;
    PrintContents ("c4 = c1 * c2", *c4);

    Collection <CTest> *c5 = *c1 - *c2;
    PrintContents ("c5 = c1 - c2", *c5);

    *c5 += *c3;
    PrintContents ("c5 += c3", *c5);

    *c5 -= *c4;
    PrintContents ("c5 -= c4", *c5);

    *c5 *= c1;
    PrintContents ("c5 *= c1", *c5);

    set2_test (c3, c3);
    set2_test (c3, c4);
    set2_test (c3, c5);

    set2_test (c4, c3);
    set2_test (c4, c4);
    set2_test (c4, c5);

    set2_test (c5, c3);
    set2_test (c5, c4);
    set2_test (c5, c5);

    delete c1;
    delete c2;
    delete c3;
    delete c4;
    delete c5;

    delete (CTest *) r1;
    delete (CTest *) r2;
    delete (CTest *) r3;
    delete (CTest *) r4;
    delete (CTest *) r5;
}

#define COUNT 4

void TestCollection (const char *str, Collection <CTest> &c)
{
    PrintInfo (str, c);

    set_test (c);

    for (int i = 0; i < COUNT; i++) {
	Ref <CTest> ref (new CTest (i));
	c.insert_element (ref);
    }

    PrintInfo (str, c);
    PrintContents ("c", c);

    for (i = 0; i < COUNT; i++) {
	Ref <CTest> ref (new CTest (1 + i));
	c.insert_element (ref);
    }

    PrintInfo (str, c);
    PrintContents ("c", c);

    Collection <CTest> *c2 = c.copy ();

    PrintInfo ("Collection <CTest> *c2 = c.copy ()", *c2);
    PrintContents ("c2", *c2);

    remove_element_at_test (c);
    replace_element_at_test (c);
    retrieve_element_at_test (c);

    select_element_test (c, "select something");
    select_test (c, "select something");

    query_test (c, "select something");
    exists_element_test (c, "select something");

    contains_element_test (c, *c2);
    contains_element_test (*c2, c);

    Stress (c);

    remove_all_test (*c2);
    delete_all_test (c);

    delete c2;
}
